#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
"""

# future
from __future__ import unicode_literals

# Django
from django.contrib import admin
from pass_manager import models


class PasswordHistoryAdmin(admin.ModelAdmin):
    list_display = ('id', 'user', 'password', 'timestamp', )
    list_display_links = ('id', 'user', )
    search_fields = ('user', )
    raw_id_fields = ('user', )
    list_per_page = 20
    ordering = ('-id', )

admin.site.register(models.PasswordHistory, PasswordHistoryAdmin)



#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
"""

# future
from __future__ import unicode_literals

from pass_manager import utils

from urllib.parse import urlparse, urlunparse
from rest_framework.response import Response
from rest_framework import status
from django.http import QueryDict

from . import settings


class ExpiredPasswordMixin(object):
    """
        This Mixin is used to check Expiry date of password.
        Also we will store user Password to maintain the history of his passwords as per `PASSWORD_HISTORY_LIFE`.
        So that we can check that user cannot use same Password as he had used in Past.
    """

    def initial(self, request, *args, **kwargs):
        if utils.is_authenticated(request.user) and not request.user.is_staff:
            next_url = utils.resolve(request.path).url_name
            # Authenticated users must be allowed to access
            # "change password" page and "log out" page.
            # even if password is expired.
            if next_url not in [settings.ACCOUNT_PASSWORD_CHANGE_REDIRECT_URL,
                                settings.ACCOUNT_LOGOUT_URL]:
                if utils.check_password_expired(request.user):
                    change_password_url = utils.reverse(settings.ACCOUNT_PASSWORD_CHANGE_REDIRECT_URL)
                    url_bits = list(urlparse(change_password_url))
                    querystring = QueryDict(url_bits[4], mutable=True)
                    url_bits[4] = querystring.urlencode(safe="/")

                    self._finalize_response(request, url_bits)

    def _finalize_response(self, request, response, *args, **kwargs):
        return Response({'detail': urlunparse(response)},
                        status=status.HTTP_307_TEMPORARY_REDIRECT)



from django.conf import settings

ACCOUNT_LOGOUT_URL = getattr(settings, 'ACCOUNT_LOGOUT_URL', '/logout/')
ACCOUNT_PASSWORD_CHANGE_REDIRECT_URL = getattr(settings, 'ACCOUNT_PASSWORD_CHANGE_REDIRECT_URL',
                                               '/password/reset/')

PASSWORD_REGEX = getattr(settings, 'PASSWORD_REGEX', '^(?=.*[A-Z])(?=.*[!@#$&*])(?=.*[0-9])(?=.*[a-z]).{6,}$')
STORE_PASSWORD_HISTORY = getattr(settings, 'STORE_PASSWORD_HISTORY', True)

# counts (Store user last 5 passwords)
PASSWORD_HISTORY_LIFE = getattr(settings, 'PASSWORD_HISTORY_LIFE', 24)

# in days
PASSWORD_EXPIRY_TIME = getattr(settings, 'PASSWORD_EXPIRY_TIME', 90)



#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
"""

# future
from __future__ import unicode_literals

# Django
from django.db import models
from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from django.utils import timezone
from .settings import PASSWORD_HISTORY_LIFE

User = settings.AUTH_USER_MODEL


def validate_password_history(obj):
    """Here we will validate Password History count against `PASSWORD_HISTORY_LIFE`.

        - A user password history stores as per `PASSWORD_HISTORY_LIFE`.
        - If the limit has been reached then we will delete oldest Password Record so that we can store latest one.

    """
    model = obj.__class__
    pass_objects = model.objects.filter(user=obj.user)

    if pass_objects.count() < PASSWORD_HISTORY_LIFE:
        return
    else:
        oldest_object = pass_objects.first()  # get oldest Password object
        oldest_object.delete()  # Delete Password object.
        return


class PasswordHistory(models.Model):
    """Contains password history for user.

        - Count of Password History depends on `PASSWORD_HISTORY_LIFE` setting.
    """

    user = models.ForeignKey(User,
                             related_name="password_history",
                             on_delete=models.CASCADE)
    password = models.CharField(
        _('Password'),
        max_length=255
    )  # encrypted password
    timestamp = models.DateTimeField(
        _('Password creation Timestamp.'),
        default=timezone.now
    )  # password creation time

    class Meta:
        verbose_name = _('User Password History')
        verbose_name_plural = _('User Passwords History')

    def __str__(self):
        return '<{user}>: {timestamp}'.format(
            user=self.user,
            timestamp=self.timestamp,
        )

    def save(self, force_insert=False, force_update=False, using=None,
             update_fields=None):
        """

        """
        if self.pk is None: # validate only for new objects
            # validate Password History
            validate_password_history(self)
        return super(PasswordHistory, self).save(force_insert=False, force_update=False, using=None,
             update_fields=None)




# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2017-09-25 06:02
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='PasswordHistory',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('password', models.CharField(max_length=255, verbose_name='Password')),
                ('timestamp', models.DateTimeField(default=django.utils.timezone.now, verbose_name='Password creation Timestamp.')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='password_history', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name_plural': 'User Passwords History',
                'verbose_name': 'User Password History',
            },
        ),
    ]



#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
"""

# future
from __future__ import unicode_literals

from urllib.parse import urlparse, urlunparse

# Django
try:
    from django.utils.deprecation import MiddlewareMixin
except ImportError:  # MiddlewareMixin is depreciated so catch this exception
    MiddlewareMixin = object

from django.http import HttpResponseRedirect, QueryDict
from pass_manager import utils

from . import settings


class ExpiredPasswordMiddleware(MiddlewareMixin):
    """
    This middleware is used to check Expiry date of password.
    Also we will store user Password to maintain the history of his passwords as per `PASSWORD_HISTORY_LIFE`.
    So that we can check that user cannot use same Password as he had used in Past.
    """

    def process_request(self, request):
        if utils.is_authenticated(request.user) and not request.user.is_staff:
            next_url = utils.resolve(request.path).url_name
            # Authenticated users must be allowed to access
            # "change password" page and "log out" page.
            # even if password is expired.
            if next_url not in [settings.ACCOUNT_PASSWORD_CHANGE_REDIRECT_URL,
                                settings.ACCOUNT_LOGOUT_URL]:

                if utils.check_password_expired(request.user):
                    change_password_url = utils.reverse(settings.ACCOUNT_PASSWORD_CHANGE_REDIRECT_URL)
                    url_bits = list(urlparse(change_password_url))
                    querystring = QueryDict(url_bits[4], mutable=True)
                    url_bits[4] = querystring.urlencode(safe="/")

                    return HttpResponseRedirect(urlunparse(url_bits))



#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
"""

# future
from __future__ import unicode_literals

import pytz
from urllib.parse import urlparse, urlunparse
from datetime import datetime, timedelta

# Django
import django

from django.http import QueryDict
from django.core.exceptions import ValidationError
from django.contrib.auth.hashers import make_password, check_password
from pass_manager.models import PasswordHistory

from . import settings

try:
    from django.core.urlresolvers import resolve, reverse, NoReverseMatch
except ImportError:
    from django.urls import resolve, reverse, NoReverseMatch


def is_authenticated(user):
    if django.VERSION >= (1, 10):
        return user.is_authenticated
    else:
        return user.is_authenticated()


def expire_middleware_as_function(request, user):
    """
    In DRF we can't use Django middleware since in DRF Authentication is handled at View layer So we will always get
    Anonymous user if we will use request.user to get current authenticated user, So we can use this function
    to check user password validity.
    """
    if not user.is_staff:
        next_url = resolve(request.path).url_name
        # Authenticated users must be allowed to access
        # "change password" page and "log out" page.
        # even if password is expired.
        if next_url not in [settings.ACCOUNT_PASSWORD_CHANGE_REDIRECT_URL,
                            settings.ACCOUNT_LOGOUT_URL,
                            ]:
            return check_password_expired(user)


def get_pass_reset_url():
    """
    :return password rest url.
    """
    change_password_url = reverse(settings.ACCOUNT_PASSWORD_CHANGE_REDIRECT_URL)
    url_bits = list(urlparse(change_password_url))
    querystring = QueryDict(url_bits[4], mutable=True)
    url_bits[4] = querystring.urlencode(safe="/")

    return urlunparse(url_bits)


def check_password_expired(user):
    """check Password expiry

    :return True if expired else False
    """
    if not settings.STORE_PASSWORD_HISTORY:
        return False

    expiry = getattr(settings, 'PASSWORD_EXPIRY_TIME')

    try:
        # get latest password info
        latest = user.password_history.latest("pk")
    except PasswordHistory.DoesNotExist:
        return True

    now = datetime.now(tz=pytz.UTC)
    expiration = latest.timestamp + timedelta(days=expiry)

    if expiration < now:
        return True
    else:
        return False


def validate_and_create_password_history(password, user):
    """
        - Validate user new password must not be as same as he have already used in Past (the history be storing)

    :param password: plaintext password
    :param user: user object
    """
    if settings.STORE_PASSWORD_HISTORY:
        # validate password history

        for item in PasswordHistory.objects.filter(user=user):
            if check_password(password, item.password):
                raise ValidationError({'detail': 'cannot use any one of your last {0} passwords'.
                                      format(settings.PASSWORD_HISTORY_LIFE)})

        PasswordHistory.objects.create(
            user=user,
            password=make_password(password)
        )



from distutils.core import setup

from setuptools import find_packages

setup(
    name='django-password-manager',
    version='0.1.2',
    description='Enable Password Force Reset & Maintain Password History',
    author='Neeraj Dhiman',
    author_email='ndhiman08@gmail.com',
    license='GPL',
    url='https://github.com/inforian/django-password-manager',
    download_url='https://github.com/inforian/django-password-manager/archive/v0.1.1.tar.gz',
    keywords=['password', 'django-password', 'django-password-manager', 'django-password-manage'],
    classifiers=[],
    packages=find_packages(),
    install_requires=[
        'Django>=1.9',
    ]
)
